/*============================================================================

                        PVK40 & AVR4PVK Example : A/D Converter

 Compiler/IDE : WinAVR

 Processor : ATMEGA64
 Clock : XT 14.74560 MHz

 Processor fuse bits:
  CKOPT: XT up to 16MHz, rail-to-rail
  SUT, CKSET: XT 3.0-8.0MHz, 16Kck + 65ms
  JTAGEN: Enabled
  M103C: Normal Mode

 Please set the DIP switches on your PVK40 board the following way:
 1) Switch off all DIP switches except:
 2) Switch on B3 LED on S11
 3) Switch on PHOTO on S9

============================================================================*/


/*
   Includes
*/
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>



/* ==========================================================================
   User definitions
*/
typedef unsigned int clock_t;
void ledch (void);
clock_t clock (void);
void timer1init (void);
void settimer1 (unsigned int speed);
void adcinit (void);
unsigned int adcsample (void);

clock_t systimer;                                     /* systimer is incremented every 33ms */
clock_t now;
clock_t lap;
unsigned int period;

#define LED_DDR        DDRA
#define LED_PORT       PORTA
#define LED_PIN        PINA
#define LED_BIT        3

#define CPER           25                             /* max delay is 1s */



/* ==========================================================================
   TIMER1 interrupt
*/
ISR (TIMER1_OVF_vect) {
  TCNT1 = period;
  systimer++;
}



/* ==========================================================================
   Read systimer
*/
clock_t clock (void) {
  clock_t temp;

  cli();
  temp = systimer;
  sei();
  return temp;
}



/* ==========================================================================
   Change the LED state
*/
void ledch (void) {
  if (LED_PIN & _BV(LED_BIT))
    LED_PORT &= ~_BV(LED_BIT);
  else
    LED_PORT |= _BV(LED_BIT);
}



/* ==========================================================================
   Initialisation of Timer1
*/
void timer1init (void) {
  TCCR1B = _BV(CS11);
  TIMSK = _BV (TOIE1);
  sei();                                              /* enable irq */
}



/* ==========================================================================
   Set value for Timer 1
*/
void settimer1 (unsigned int speed) {
  cli();
  period = speed;
  sei();
}



/* ==========================================================================
   Initialisation of AD converter
*/
void adcinit (void) {
  ADMUX = _BV(REFS0) | _BV(ADLAR) | _BV(MUX0);        /* AVCC; Right Adjust; ADC1 */
  ADCSRA = _BV(ADEN) | _BV(ADPS2);                    /* Enable ADC; Prescaler = 16 */
}



/* ==========================================================================
   ADC refresh
*/
unsigned int adcsample (void) {
  ADCSRA |= _BV(ADSC);                                /* Start Conversion */
  while (!(ADCSRA & _BV(ADIF)));                      /* Wait until done */

  return ADC;
}



/* ==========================================================================
   Main function
*/
int main (void) {
  unsigned int speed;

  LED_DDR |= _BV(LED_BIT);
  timer1init();
  adcinit();

  for(;;) {
    now = clock();
    if ((now - lap) > CPER) {
      lap += CPER;
      ledch();
      speed = adcsample(); 
      settimer1(speed);    //will change the timer1 counting register start value in accordance with the measured value 
    }
  }
}
